﻿using EvaPOS_API_FRAME.DTO;
using EvaPOS_API_FRAME.RespuestasXML;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using Serilog;
using EvaPOS_API_FRAME.DTO.Intercambio;
using System.Globalization;
using SCOGateway.POSGk;
using RestSharp;
using gatewayGK.POSGk;

namespace EvaPOS_API_FRAME.Comandos
{
    ///<summary>
    /// Procesa solicitudes del primer item agregado.
    /// </summary>
    public class AddItemRequestCmdGk : IComando
    {
        public string Referencia { get; set; } = "scsns:AddItem";
        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public AddItemRequest Request { get; private set; }

        /// <summary>
        /// Procesa y responde ReportStatusEventsRequest.
        /// </summary>
        /// 

        public const string UrlItem = ConstantesGK.UrlBaseMaqueta + "/com.gk_software.pos.api.service.transaction.LineItemService/createAndAddInvoicePaymentLineItem";
        /// <summary>
        /// Url servicio subtotal Gk.
        /// </summary>
        public Respuestas Ejecutar()
        {
            Log.Debug("Cmd AddItemRequestCmdGk ejecutado. AddItemRequest : {AddItemRequest}", Request.RequestID);
            //int requestIdGuardado = RequestIdGuardado();
            POSBCStatusEvent posbcStatus = null;
            //Variables para manejar las respuestas de error - (item con peso, no se encuentra el item,item requiere cantidad)
            AddItemResponseError addItemResponseError = null;
            Respuestas respuesta = null;
            Venta venta = null;
            //Variables del adaptador de entrada - addItem
            int requestId = Request.RequestID;
            long barCode = 0;
            int quantity = Request.ItemIdentifier.Quantity == 0 ? 1 : Request.ItemIdentifier.Quantity;
            bool cancelItem = Request.ItemIdentifier.VoidFlag;
            double peso = Request.ItemIdentifier.ScaleWeight;
            //Logica para validar el valor del codigo de barras
            //Si es un item de busqueda o imagen, trae el valor KeyedItemID, si es escaneado trae barcode
            if (Request.ItemIdentifier.BarCode != null)
                barCode = Request.ItemIdentifier.BarCode.ScanDataLabel;
            else
                barCode = Request.ItemIdentifier.KeyedItemID;

            string barcodeString = barCode.ToString();

            var item = new RestClient(UrlItem);
            var request = new RestRequest(UrlItem, Method.Post);
            var items = new AddItemReq { ItemID = barcodeString };
            request.AddJsonBody(items);

            // Ejecuta invocación sincrónica, deserializando respuesta según el tipo indicado en la invocación del Post<T>.
            var itemlResp = item.Post<AddItemResp>(request) ?? throw new ApplicationException("Servicio POST 'AddItems' retorna nulos.");
            Log.Debug($"GkPruebas: Respuesta Cliente - POST - response {itemlResp}");

            string tomarNombreCorto = "";
            //Valores que se mandan en chec
            var date = DateTime.Now.ToString("dd/MM/yyyy");
            CultureInfo culture = new CultureInfo("en-US");
            culture.DateTimeFormat.AMDesignator = "AM";
            culture.DateTimeFormat.PMDesignator = "PM";
            var time = DateTime.Now.ToString("hh:mm tt", culture);

            if (itemlResp != null)
            {
                TransactionStatusEvent transactionStatusEvent = new TransactionStatusEvent(1, TipoMensaje.Event, requestId, "TRANSACTION_START", 1, "regularSale", "sales", date, time);
                POSReceiptEvent pOSReceiptEvent = new POSReceiptEvent(1, TipoMensaje.Event, requestId, "Customer", 0, "Body", requestId, 1, "center", "LineItem", "ItemSale", itemlResp.ItemID + "   " + itemlResp.Name + "      " + "1" + "      " + itemlResp.RegularPriceAmount, "Add");
                POSReceiptEventHeaderResponse posReceiptEventHeader = new POSReceiptEventHeaderResponse(1, TipoMensaje.Event, requestId);
                TotalsEventResponse totalEvent = new TotalsEventResponse(1, TipoMensaje.Event, requestId, itemlResp.RegularPriceAmount.ToString() , itemlResp.RegularPriceAmount.ToString(), "0.18", "3.67", "0.00", "0.00", "0.00", "0.00", "0.00", itemlResp.Quantity, 0, "0.00", "0.00");
                AddItemResponse itemResponse = new AddItemResponse(1, TipoMensaje.Resp, requestId, itemlResp.Name, "false", "false", "true", itemlResp.ItemID, "ScannedItemCode", itemlResp.RegularPriceAmount.ToString(), itemlResp.Quantity, itemlResp.RegularPriceAmount.ToString(), 0, "0.00", 0, 1, 0, "unit", "false", 0, "false", "false", "true", "true");
                respuesta = new Respuestas { transactionStatusEvent, posReceiptEventHeader, pOSReceiptEvent, totalEvent, itemResponse };
            }
            else
            {
                addItemResponseError = new AddItemResponseError(1, TipoMensaje.Resp, requestId, "Articulo no encontrado", "ITEM_NOT_FOUND");
                return new Respuestas { addItemResponseError };
            }

           

            

            return respuesta;
        }


        public IComando CreaCopia()
        {
            return (AddItemRequestCmd)this.MemberwiseClone();
        }

        public void CargaDTO(DTOBase addItemRequestDTO)
        {
            Request = (AddItemRequest)addItemRequestDTO;
        }

    }
}
