﻿using EvaPOS_API_FRAME.Comandos;
using EvaPOS_API_FRAME.DTO;
using EvaPOS_API_FRAME.RespuestasXML.Printer;
using EvaPOS_API_FRAME.RespuestasXML;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using Serilog;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SCOGateway.POSGk;
using gatewayGK.POSGk;
using RestSharp;

namespace gatewayGK.ComandosGkPruebas
{
    public class AddTenderCmdGk : IComando
    {
        public string Referencia { get; set; } = "scsns:AddTender";
        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public AddTenderRequest Request { get; private set; }

        /// <summary>
        /// Procesa y responde eventos del pago con tarjeta de debito
        /// </summary>
        public const string UrlTender = ConstantesGK.UrlBaseMaqueta + "/com.gk_software.pos.api.service.payment.PaymentService/addCashChangeAmountAlreadyPaidExtensions";
        /// <summary>
        /// Url servicio Tender Gk.
        /// </summary>

        public Respuestas Ejecutar()
        {
            //instanciar xml de respuestas
            TransactionStatusEvent transactionStatusEvent = null;
            POSReceiptEventDebit posbcEvent1 = null;
            POSReceiptEvent posbcEvent3 = null;
            POSReceiptEvent posbcEvent4 = null;
            TotalsEventResponse totalEvent = null;
            AddTenderDebitResponse addTender = null;
            AddTenderResponseError cancelActionResponse = null;
            POSBCStatusEventPrinterSingle posbcStatusPrinter = null;

            Log.Debug("Cmd AddTenderDebitDTO ejecutado. AddTenderDebit : {AddTenderDebit}", Request.RequestID);
            int requestId = Request.RequestID;
            ReportStatusEventsRequestCmd reportStatusEventsRequestCmd = new ReportStatusEventsRequestCmd();
            AddItemRequestCmd addItemRequestCmd = new AddItemRequestCmd();
            Respuestas respuestasChec = null;

            var tender = new RestClient(UrlTender);
            var request = new RestRequest(UrlTender, Method.Post);
            var tenders = new AddTenderReq { TenderClassCode = "012" };
            request.AddJsonBody(tenders);

            // Ejecuta invocación sincrónica, deserializando respuesta según el tipo indicado en la invocación del Post<T>.
            var tenderResp = tender.Post<AddTenderResp>(request) ?? throw new ApplicationException("Servicio POST 'AddTenderResp' retorna nulos.");
            Log.Debug($"GkPruebas: Respuesta Cliente - POST - response {tenderResp}");

            var date = DateTime.Now.ToString("MM/dd/yyyy");
            CultureInfo culture = new CultureInfo("en-US");
            culture.DateTimeFormat.AMDesignator = "AM";
            culture.DateTimeFormat.PMDesignator = "PM";
            var time = DateTime.Now.ToString("hh:mm tt", culture);

            if(tenderResp != null)
            {
                posbcEvent3 = new POSReceiptEvent(1, TipoMensaje.Event, requestId, "Customer", 0, "Trailer", requestId, 1, "center", "Trailer", "WorkstationInfo", date + time, "Add");
                posbcEvent4 = new POSReceiptEvent(1, TipoMensaje.Event, requestId, "Customer", 0, "Trailer", requestId, 1, "center", "Trailer", "WorkstationInfo", date + time, "Add");
                posbcEvent1 = new POSReceiptEventDebit(1, TipoMensaje.Event, requestId, date + time);
                totalEvent = new TotalsEventResponse(1, TipoMensaje.Event, requestId, "3200", "3200", "3200", "3200", "0.00", "0.00", "0.00", "0.00", "0.00", 5200, 0, "0.00", "0.00");
                transactionStatusEvent = new TransactionStatusEvent(1, TipoMensaje.Event, requestId, "TRANSACTION_END", 1, "regularSale", "sales", date, time);

                addTender = new AddTenderDebitResponse(1, TipoMensaje.Resp, requestId, "false", "false", "false", tenderResp.PayedAmount.ToString(), "debit", "true", "9999", -1, 00350383, tenderResp.SequenceNumber, 0, tenderResp.TransactionPayed.ToString(), "approved", "false");
                respuestasChec = new Respuestas { posbcEvent3, posbcEvent4, posbcEvent1, totalEvent, transactionStatusEvent, addTender };
            }
            else
            {
                cancelActionResponse = new AddTenderResponseError(1, TipoMensaje.Resp, requestId, "Cancelar", "UNSUPPORTED_OPERATION");
                respuestasChec = new Respuestas { cancelActionResponse };
            }

            

            return respuestasChec;
        }

        public IComando CreaCopia()
        {
            return (AddTenderDebitCmd)this.MemberwiseClone();
        }
        public void CargaDTO(DTOBase AddTenderRequestDebitDTO)
        {
            Request = (AddTenderRequest)AddTenderRequestDebitDTO;
        }
    }
}
